// SVAEClone.c
//
// 7Edit 3.1d1. Original version by Jon Lansdell and Nigel Humphreys.
// 3.1 updates by Greg Sutton.
// Apple Computer Inc 1995, all rights reserved.

#include "SVAEClone.h"

#include "SVEditAEUtils.h"
#include "SVAECreate.h"
#include "SVAEGetData.h"


#pragma segment AppleEvent


// Clone an object - only handles text objects
	 
	 
pascal OSErr	DoClone(const AppleEvent *theAppleEvent, AppleEvent *reply, long refcon)
{
#pragma unused (reply, refcon)

	AEDesc		directObj = {typeNull, NULL},
				insertHereDesc = {typeNull, NULL},
				resultDesc = {typeNull, NULL};
	OSErr		err;

	err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard, &directObj);
	if (noErr != err) goto done;

	err = AEGetParamDesc(theAppleEvent, keyAEInsertHere, typeWildCard, &insertHereDesc);
	if (noErr != err) goto done;

	err = GotRequiredParams(theAppleEvent);
	if (noErr != err) goto done;
	
	err = CloneDesc(&directObj, &insertHereDesc, &resultDesc);
	if (noErr != err) goto done;
	
	err = AddResultToReply(&resultDesc, reply, err);

done:
	if (directObj.dataHandle)
		AEDisposeDesc(&directObj);
	if (insertHereDesc.dataHandle)
		AEDisposeDesc(&insertHereDesc);
		
	return(err);
}

OSErr	CloneTextDesc(AEDesc* textDesc, AEDesc* insertHereDesc, AEDesc* result)
{
	TextToken		aTextToken;
	Size			actualSize;
	AEDesc			dataDesc = {typeNull, NULL},
					propertyDesc = {typeNull, NULL};	// Use to pass no properties
	OSErr			err;

	if (typeMyText != textDesc->descriptorType)
		return(errAETypeError);
		
	GetRawDataFromDescriptor(textDesc, (Ptr)&aTextToken, sizeof(aTextToken), &actualSize);
	
	err = GetTextTextProperty(&aTextToken, &dataDesc);
	if (noErr != err) goto done;
	
	err = CreateText(cText, &dataDesc, insertHereDesc, &propertyDesc, result);

done:
	if (dataDesc.dataHandle)
		AEDisposeDesc(&dataDesc);

	return(err);
}

OSErr	CloneDesc(AEDesc* aDesc, AEDesc* insertHereDesc, AEDesc* result)
{
	AEDesc		cloneDesc = {typeNull, NULL},
				textDesc = {typeNull, NULL};
	OSErr		err;
	
	if (typeObjectSpecifier == aDesc->descriptorType)
		err = AEResolve(aDesc, kAEIDoMinimum, &cloneDesc);
	else
		err = AEDuplicateDesc(aDesc, &cloneDesc);
		
	if (noErr != err) goto done;
	
	switch (cloneDesc.descriptorType)
	{
		case typeAEList:
			err = errAETypeError;
			// We can't handle copying more than one item to the scrap
			break;
			
		default:
			err = AECoerceDesc(&cloneDesc, typeMyText, &textDesc);
			if (noErr != err) goto done;
			err = CloneTextDesc(&textDesc, insertHereDesc, result);
	}
	
done:
	if (cloneDesc.dataHandle)
		AEDisposeDesc(&cloneDesc);
	if (textDesc.dataHandle)
		AEDisposeDesc(&textDesc);
	
	return(err);
}