// SVAEObjectsExist.c
//
// 7Edit 3.1d1. Original version by Jon Lansdell and Nigel Humphreys.
// 3.1 updates by Greg Sutton.
// Apple Computer Inc 1995, all rights reserved.

#include "SVAEObjectsExist.h"

#include "SVEditAEUtils.h"


#pragma segment AppleEvent


// -----------------------------------------------------------------------
//	Name: 		DoObjectsExist
//	Purpose:	Handles the kAEDoObjectsExist AppleEvent. Basically just
//				tries to resolve the object to see if it exists.
// -----------------------------------------------------------------------

pascal OSErr DoObjectsExist(const AppleEvent	*theAppleEvent,
									AppleEvent	*reply, 
									long		handlerRefCon)
{
#pragma unused (handlerRefCon)
 
	AEDesc        	directObject = {typeNull, NULL},
					directDesc = {typeNull, NULL},
					replyDesc = {typeNull, NULL};
	long			itemCount;
	Boolean       	exists;
	OSErr         	existsErr,
					err;
			
	err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard, &directObject);

	err = GotRequiredParams(theAppleEvent);
	if (noErr != err) goto done;

	if (typeNull == directObject.descriptorType)
		exists = true;		// Yes, our application does exist?????
	else
	{
		existsErr = AEResolve(&directObject, kAEIDoMinimum, &directDesc);
		
		if (noErr == existsErr)
		{
			if (typeAEList == directDesc.descriptorType)	// If it's a list then objects
			{												// exist only if the list
				err = AECountItems(&directDesc, &itemCount);// is not empty.
				if (noErr != err) goto done;

				if (itemCount)
					exists = true;
				else
					exists = false;
			}
			else
				exists = true;								// Otherwise it resolved
		}													// so it exists.
		else
			exists = false;
	}
	
	err = AECreateDesc(typeBoolean, (Ptr)&exists, sizeof(exists), &replyDesc);
	if (noErr != err) goto done;
	
	err = AddResultToReply(&replyDesc, reply, err);
	
done:
	if (directObject.dataHandle)
		AEDisposeDesc(&directObject);
	if (directDesc.dataHandle)
		AEDisposeDesc(&directDesc);
	if (replyDesc.dataHandle)
		AEDisposeDesc(&replyDesc);
		
	return(err);
} // DoObjectsExist
